searchData={"items":[{"type":"behaviour","title":"ConsumerSupervisor","doc":"A supervisor that starts children as events flow in.\n\nA `ConsumerSupervisor` can be used as the consumer in a `GenStage` pipeline.\nA new child process will be started per event, where the event is appended\nto the arguments in the child specification.\n\nA `ConsumerSupervisor` can be attached to a producer by returning\n`:subscribe_to` from `c:init/1` or explicitly with `GenStage.sync_subscribe/3`\nand `GenStage.async_subscribe/2`.\n\nOnce subscribed, the supervisor will ask the producer for `:max_demand` events\nand start child processes as events arrive. As child processes terminate, the\nsupervisor will accumulate demand and request more events once `:min_demand`\nis reached. This allows the `ConsumerSupervisor` to work similar to a pool,\nexcept a child process is started per event. The minimum amount of concurrent\nchildren per producer is specified by `:min_demand` and the maximum is given\nby `:max_demand`.","ref":"ConsumerSupervisor.html"},{"type":"behaviour","title":"Example - ConsumerSupervisor","doc":"Let's define a GenStage consumer as a `ConsumerSupervisor` that subscribes\nto a producer named `Producer` and starts a new process for each event\nreceived from the producer. Each new process will be started by calling\n`Printer.start_link/1`, which simply starts a task that will print the\nincoming event to the terminal.\n\n    defmodule Consumer do\n      use ConsumerSupervisor\n\n      def start_link(arg) do\n        ConsumerSupervisor.start_link(__MODULE__, arg)\n      end\n\n      def init(_arg) do\n        # Note: By default the restart for a child is set to :permanent\n        # which is not supported in ConsumerSupervisor. You need to explicitly\n        # set the :restart option either to :temporary or :transient.\n        children = [%{id: Printer, start: {Printer, :start_link, []}, restart: :transient}]\n        opts = [strategy: :one_for_one, subscribe_to: [{Producer, max_demand: 50}]]\n        ConsumerSupervisor.init(children, opts)\n      end\n    end\n\nThen in the `Printer` module:\n\n    defmodule Printer do\n      def start_link(event) do\n        # Note: this function must return the format of `{:ok, pid}` and like\n        # all children started by a Supervisor, the process must be linked\n        # back to the supervisor (if you use `Task.start_link/1` then both\n        # these requirements are met automatically)\n        Task.start_link(fn ->\n          IO.inspect({self(), event})\n        end)\n      end\n    end\n\nSimilar to `Supervisor`, `ConsumerSupervisor` also provides `start_link/3`,\nwhich allows developers to start a supervisor with the help of a callback\nmodule.","ref":"ConsumerSupervisor.html#module-example"},{"type":"behaviour","title":"Name Registration - ConsumerSupervisor","doc":"A supervisor is bound to the same name registration rules as a `GenServer`.\nRead more about it in the `GenServer` docs.","ref":"ConsumerSupervisor.html#module-name-registration"},{"type":"function","title":"ConsumerSupervisor.count_children/1","doc":"Returns a map containing count values for the supervisor.\n\nThe map contains the following keys:\n\n  * `:specs` - always `1` as consumer supervisors have a single specification\n\n  * `:active` - the count of all actively running child processes managed by\n    this supervisor\n\n  * `:supervisors` - the count of all supervisors whether or not the child\n    process is still alive\n\n  * `:workers` - the count of all workers, whether or not the child process\n    is still alive","ref":"ConsumerSupervisor.html#count_children/1"},{"type":"callback","title":"ConsumerSupervisor.init/1","doc":"Callback invoked to start the supervisor and during hot code upgrades.","ref":"ConsumerSupervisor.html#c:init/1"},{"type":"callback","title":"Options - ConsumerSupervisor.init/1","doc":"* `:strategy` - the restart strategy option. Only `:one_for_one`\n    is supported by consumer supervisors.\n\n  * `:max_restarts` - the maximum amount of restarts allowed in\n    a time frame. Defaults to 3 times.\n\n  * `:max_seconds` - the time frame in which `:max_restarts` applies\n    in seconds. Defaults to 5 seconds.\n\n  * `:subscribe_to` - a list of producers to subscribe to. Each element\n    represents the producer or a tuple with the producer and the subscription\n    options, for example, `[Producer]` or `[{Producer, max_demand: 20, min_demand: 10}]`.","ref":"ConsumerSupervisor.html#c:init/1-options"},{"type":"function","title":"ConsumerSupervisor.init/2","doc":"Receives a template to initialize and a set of options.\n\nThis is typically invoked at the end of the `c:init/1` callback of module-based supervisors.\n\nThis function returns a the child specification and the supervisor flags.","ref":"ConsumerSupervisor.html#init/2"},{"type":"function","title":"Examples - ConsumerSupervisor.init/2","doc":"Using the child specification changes introduced in Elixir 1.5:\n\n    defmodule MyConsumerSupervisor do\n      use ConsumerSupervisor\n\n      def start_link(arg) do\n        ConsumerSupervisor.start_link(__MODULE__, arg)\n      end\n\n      def init(_arg) do\n        ConsumerSupervisor.init([MyConsumer], strategy: :one_for_one, subscribe_to: MyProducer)\n      end\n    end","ref":"ConsumerSupervisor.html#init/2-examples"},{"type":"type","title":"ConsumerSupervisor.option/0","doc":"Options used by the `start*` functions","ref":"ConsumerSupervisor.html#t:option/0"},{"type":"function","title":"ConsumerSupervisor.start_child/2","doc":"Starts a child in the consumer supervisor.\n\nThe child process will be started by appending the given list of\n`args` to the existing function arguments in the child specification.\n\nThis child is started separately from any producer and does not\ncount towards the demand of any of them.\n\nIf the child process starts, function returns `{:ok, child}` or\n`{:ok, child, info}`, the pid is added to the supervisor, and the\nfunction returns the same value.\n\nIf the child process start function returns `:ignore`, an error tuple,\nor an erroneous value, or if it fails, the child is discarded and\n`:ignore` or `{:error, error}` where `error` is a term containing\ninformation about the error is returned.","ref":"ConsumerSupervisor.html#start_child/2"},{"type":"function","title":"ConsumerSupervisor.start_link/2","doc":"Starts a supervisor with the given children.\n\nA strategy is required to be given as an option. Furthermore,\nthe `:max_restarts`, `:max_seconds`, and `:subscribe_to`\nvalues can be configured as described in the documentation for the\n`c:init/1` callback.\n\nThe options can also be used to register a supervisor name.\nThe supported values are described under the \"Name Registration\"\nsection in the `GenServer` module docs.\n\nThe child processes specified in `children` will be started by appending\nthe event to process to the existing function arguments in the child specification.\n\nNote that the consumer supervisor is linked to the parent process\nand will exit not only on crashes but also if the parent process\nexits with `:normal` reason.","ref":"ConsumerSupervisor.html#start_link/2"},{"type":"function","title":"ConsumerSupervisor.start_link/3","doc":"Starts a consumer supervisor module with the given `args`.\n\nTo start the supervisor, the `c:init/1` callback will be invoked in the given\nmodule, with `args` passed to it. The `c:init/1` callback must return a\nsupervision specification which can be created with the help of the\n`Supervisor` module.\n\nIf the `c:init/1` callback returns `:ignore`, this function returns\n`:ignore` as well and the supervisor terminates with reason `:normal`.\nIf it fails or returns an incorrect value, this function returns\n`{:error, term}` where `term` is a term with information about the\nerror, and the supervisor terminates with reason `term`.\n\nThe `:name` option can also be given in order to register a supervisor\nname. The supported values are described under the \"Name Registration\"\nsection in the `GenServer` module docs.","ref":"ConsumerSupervisor.html#start_link/3"},{"type":"function","title":"ConsumerSupervisor.terminate_child/2","doc":"Terminates the given child pid.\n\nIf successful, the function returns `:ok`. If there is no\nsuch pid, the function returns `{:error, :not_found}`.","ref":"ConsumerSupervisor.html#terminate_child/2"},{"type":"function","title":"ConsumerSupervisor.which_children/1","doc":"Returns a list with information about all children.\n\nNote that calling this function when supervising a large number\nof children under low memory conditions can cause an out of memory\nexception.\n\nThis function returns a list of tuples containing:\n\n  * `id` - as defined in the child specification but is always\n    set to `:undefined` for consumer supervisors\n\n  * `child` - the pid of the corresponding child process or the\n    atom `:restarting` if the process is about to be restarted\n\n  * `type` - `:worker` or `:supervisor` as defined in the child\n    specification\n\n  * `modules` - as defined in the child specification","ref":"ConsumerSupervisor.html#which_children/1"},{"type":"behaviour","title":"GenStage","doc":"Stages are data-exchange steps that send and/or receive data\nfrom other stages.\n\nWhen a stage sends data, it acts as a **producer**. When it receives\ndata, it acts as a **consumer**. Stages may take both producer and\nconsumer roles at once, acting as **consumer producers**.","ref":"GenStage.html"},{"type":"behaviour","title":"Stage types - GenStage","doc":"Besides taking both producer and consumer roles, a stage may be\ncalled \"source\" if it only produces items or called \"sink\" if it\nonly consumes items.\n\nFor example, imagine the stages below where A sends data to B\nthat sends data to C:\n\n    [A] -> [B] -> [C]\n\nwe conclude that:\n\n  * A is only a producer (and therefore a source)\n  * B is both producer and consumer\n  * C is only a consumer (and therefore a sink)\n\nAs we will see in the upcoming \"Examples\" section, we must\nspecify the type of the stage when we implement it.\n\nTo start the flow of events, we always subscribe consumers to\nproducers. Once the communication channel between consumers and producers\nis established, consumers will ask producers for events.\nWe typically say that the consumer is sending demand upstream.\nOnce demand arrives, the producer will emit events, never\nemitting more events than the consumer asked for. This provides\na back-pressure mechanism.\n\nA consumer may have multiple producers and a producer may have\nmultiple consumers. When a consumer asks for data to many producers, each producer\nis handled separately, with its own demand. When a producer\nreceives demand and sends data to multiple consumers, the demand\nis tracked and the events are sent by a dispatcher. This allows\nproducers to send data using different \"strategies\". See\n`GenStage.Dispatcher` for more information.","ref":"GenStage.html#module-stage-types"},{"type":"behaviour","title":"Example - GenStage","doc":"Let's define the simple pipeline below:\n\n    [A] -> [B] -> [C]\n\nwhere A is a producer that will emit items starting from 0,\nB is a producer-consumer that will receive those items and\nmultiply them by a given number and C will receive those events\nand print them to the terminal.\n\nLet's start with A. Since A is a producer, its main\nresponsibility is to receive demand and generate events.\nThose events may be in memory or an external queue system.\nFor simplicity, let's implement a simple counter starting\nfrom a given value of `counter` received on `init/1`:\n\n    defmodule A do\n      use GenStage\n\n      def start_link(number) do\n        GenStage.start_link(A, number)\n      end\n\n      def init(counter) do\n        {:producer, counter}\n      end\n\n      def handle_demand(demand, counter) when demand > 0 do\n        # If the counter is 3 and we ask for 2 items, we will\n        # emit the items 3 and 4, and set the state to 5.\n        events = Enum.to_list(counter..counter+demand-1)\n        {:noreply, events, counter + demand}\n      end\n    end\n\nB is a consumer-producer. This means it does not explicitly\nhandle the demand because the demand is always forwarded to\nits producer. Once A receives the demand from B, it will send\nevents to B which will be transformed by B as desired. In\nour case, B will receive events and multiply them by a number\ngiven on initialization and stored as the state:\n\n    defmodule B do\n      use GenStage\n\n      def start_link(multiplier) do\n        GenStage.start_link(B, multiplier)\n      end\n\n      def init(multiplier) do\n        {:producer_consumer, multiplier}\n      end\n\n      def handle_events(events, _from, multiplier) do\n        events = Enum.map(events, & &1 * multiplier)\n        {:noreply, events, multiplier}\n      end\n    end\n\nC will finally receive those events and print them every second\nto the terminal:\n\n    defmodule C do\n      use GenStage\n\n      def start_link(_opts) do\n        GenStage.start_link(C, :ok)\n      end\n\n      def init(:ok) do\n        {:consumer, :the_state_does_not_matter}\n      end\n\n      def handle_events(events, _from, state) do\n        # Wait for a second.\n        Process.sleep(1000)\n\n        # Inspect the events.\n        IO.inspect(events)\n\n        # We are a consumer, so we would never emit items.\n        {:noreply, [], state}\n      end\n    end\n\nNow we can start and connect them:\n\n    {:ok, a} = A.start_link(0)  # starting from zero\n    {:ok, b} = B.start_link(2)  # multiply by 2\n    {:ok, c} = C.start_link([]) # state does not matter\n\n    GenStage.sync_subscribe(c, to: b)\n    GenStage.sync_subscribe(b, to: a)\n\nTypically, we subscribe from bottom to top. Since A will\nstart producing items only when B connects to it, we want this\nsubscription to happen when the whole pipeline is ready. After\nyou subscribe all of them, demand will start flowing upstream and\nevents downstream.","ref":"GenStage.html#module-example"},{"type":"behaviour","title":"Usage guidelines - GenStage","doc":"As you get familiar with GenStage, developers may tend to create layers\nof stages, such as A, B and C, for achieving concurrency. For example,\nstage A does step 1 in your company workflow, stage B does step 2 and\nso forth. That's an anti-pattern.\n\nThe same guideline that applies to processes also applies to GenStage:\nuse processes/stages to model runtime properties, such as concurrency and\ndata-transfer, and not for code organization or domain design purposes.\nFor the latter, you should use modules and functions.\n\nIf your domain has to process the data in multiple steps, you should write\nthat logic in separate modules and not directly in a `GenStage`. You only add\nstages according to runtime needs, typically when you need to provide back-\npressure or leverage concurrency. This way you are free to experiment with\ndifferent `GenStage` pipelines without touching your business rules.\n\nIn particular, if your logic has three distinct steps, instead of starting\nthree different stages for each step, it may be best to start multiple\ninstances of a single stage that executes all steps. Instead of this:\n\n    [Producer Stage] -> [Stage Step 1] -> [Stage Step 2] -> [Stage Step 3]\n\nYou should rather have this:\n\n                 [Consumer Step 1 + Step 2 + Step 3]\n                /\n    [Producer]->-[Consumer Step 1 + Step 2 + Step 3]\n                \\\n                 [Consumer Step 1 + Step 2 + Step 3]\n\nThe benefit of this approach is that you can scale the code based on the machine\nresources and runtime needs rather than the number of steps during development.\n\nFinally, if you don't need back-pressure at all and you just need to process\ndata that is already in-memory in parallel, a simpler solution is available\ndirectly in Elixir via `Task.async_stream/2`. This function consumes a stream\nof data, with each entry running in a separate task. The maximum number of tasks\nis configurable via the `:max_concurrency` option.","ref":"GenStage.html#module-usage-guidelines"},{"type":"behaviour","title":"Demand - GenStage","doc":"When implementing consumers, we often set the `:max_demand` and\n`:min_demand` on subscription. The `:max_demand` specifies the\nmaximum amount of events that must be in flow while the `:min_demand`\nspecifies the minimum threshold to trigger for more demand. The consumer\nwill never ask the producer for more than `:max_demand` events at a time.\nWhen the producer emits enough events for the consumer's demand to go\ndown to `:min_demand`, only then the consumer will ask for more events\n(in other words, it will \"send more demand upstream\").\n\nAn example with numbers will help clarify this. Say that `:max_demand` is\n`1000` and `:min_demand` is `750`. Initially, the consumer asks for `1000`\nevents to the producer. Say that the producer produces `100` events at a time.\nThe first \"batch\" of `100` events goes to the consumer and the consumer's demand\ngoes down to `900`. Another batch is produced and the demand goes down to `800`.\nAt this point, the producer has not been asked for more events yet since the demand\ndidn't go below `:min_demand` yet. When the producer produces the next batch of\n`100` events, the consumer will process `50` events and the demand reaches the\nminimum of `750`. The consumer sends `250` demand upstream (which is up to `:max_demand`).\nof `250` to reach `:min_demand` again, and then consume the `50` events remaining.\n`c:handle_demand/2` will be called on the producer with a demand of `250`.\n\nIn the example above, B is a `:producer_consumer` and therefore\nacts as a buffer. Getting the proper demand values in B is\nimportant: making the buffer too small may make the whole pipeline\nslower, making the buffer too big may unnecessarily consume\nmemory.\n\nWhen such values are applied to the stages above, it is easy\nto see the producer works in batches. The producer A ends-up\nemitting batches of 50 items which will take approximately\n50 seconds to be consumed by C, which will then request another\nbatch of 50 items.\n\n### `init` and `:subscribe_to`\n\nIn the example above, we have started the processes A, B, and C\nindependently and subscribed them later on. But most often it is\nsimpler to subscribe a consumer to its producer on its `c:init/1`\ncallback. This way, if the consumer crashes, restarting the consumer\nwill automatically re-invoke its `c:init/1` callback and resubscribe\nit to the producer.\n\nThis approach works as long as the producer can be referenced when\nthe consumer starts - such as by name for a named process. For example,\nif we change the process `A` and `B` to be started as follows:\n\n    # Let's call the stage in module A as A\n    GenStage.start_link(A, 0, name: A)\n    # Let's call the stage in module B as B\n    GenStage.start_link(B, 2, name: B)\n    # No need to name consumers as they won't be subscribed to\n    GenStage.start_link(C, :ok)\n\nWe can now change the `c:init/1` callback for C to the following:\n\n    def init(:ok) do\n      {:consumer, :the_state_does_not_matter, subscribe_to: [B]}\n    end\n\nSubscription options as outlined in `sync_subscribe/3` can also be\ngiven by making each subscription a tuple, with the process name or\npid as first element and the options as second:\n\n    def init(:ok) do\n      {:consumer, :the_state_does_not_matter, subscribe_to: [{B, options}]}\n    end\n\nSimilarly, we should change `B` to subscribe to `A` on `c:init/1`. Let's\nalso set `:max_demand` to 10 when we do so:\n\n    def init(number) do\n      {:producer_consumer, number, subscribe_to: [{A, max_demand: 10}]}\n    end\n\nAnd we will no longer need to call `sync_subscribe/2`.\n\nAnother advantage of using `:subscribe_to` is that it makes it straight-forward\nto leverage concurrency by simply starting multiple consumers that subscribe\nto their producer (or producer-consumer). This can be done in the example above\nby simply calling start link multiple times:\n\n    # Start 4 consumers\n    GenStage.start_link(C, :ok)\n    GenStage.start_link(C, :ok)\n    GenStage.start_link(C, :ok)\n    GenStage.start_link(C, :ok)\n\nIn a supervision tree, this is often done by starting multiple workers. Typically\nwe update each `start_link/1` call to start a named process:\n\n    def start_link(number) do\n      GenStage.start_link(A, number, name: A)\n    end\n\nAnd the same for module `B`:\n\n    def start_link(number) do\n      GenStage.start_link(B, number, name: B)\n    end\n\nModule `C` does not need to be updated because it won't be subscribed to.\nThen we can define our supervision tree like this:\n\n    children = [\n      {A, 0},\n      {B, 2},\n      Supervisor.child_spec({C, []}, id: :c1),\n      Supervisor.child_spec({C, []}, id: :c2),\n      Supervisor.child_spec({C, []}, id: :c3),\n      Supervisor.child_spec({C, []}, id: :c4)\n    ]\n\n    Supervisor.start_link(children, strategy: :rest_for_one)\n\nHaving multiple consumers is often the easiest and simplest way to leverage\nconcurrency in a GenStage pipeline, especially if events can be processed out\nof order.\n\nAlso note that we set the supervision strategy to `:rest_for_one`. This is\nimportant. Consider an alternative case where `:one_for_one` is used. If the\nproducer `A` terminates, all of the other processes will terminate too, since\nthey are consuming events produced by `A`. In this scenario, the supervisor will\nsee multiple processes shutting down at the same time, and conclude there are\ntoo many failures in a short interval. However, if the strategy is\n`:rest_for_one`, the supervisor will shut down the rest of tree, and already\nexpect the remaining processes to fail. One downside of `:rest_for_one` though\nis that if a `C` process dies, any other `C` process after it will die too.\nYou can solve this by putting them under their own supervisor if desired.\n\nAnother alternative to the scenario above is to use a `ConsumerSupervisor`\nfor consuming the events instead of N consumers. The `ConsumerSupervisor`\nwill communicate with the producer respecting the back-pressure properties\nand start a separate supervised process per event. The number of children\nconcurrently running in a `ConsumerSupervisor` is at most `max_demand` and\nthe average amount of children is `(max_demand + min_demand) / 2`.","ref":"GenStage.html#module-demand"},{"type":"behaviour","title":"Buffering - GenStage","doc":"In many situations, mismatches might happen between how many events can be produced\nand how many events can be consumed. In those cases, we usually need to *buffer*\nsome things. Let's explore the possible scenarios.\n\nIn the first scenario, producers may attempt to emit events while no consumers\nhave yet subscribed. Alternatively, producers may produce more events than the\nconsumers' demand asked for. In these cases, producers will have to buffer events\nuntil a consumer is available or consumers have enough demand again.\n\nIn the second scenario, consumers may ask producers for events that are not\nyet available. In this case, producers have to buffer consumer demand\nuntil new events can be produced.\n\nAs we will see next, buffering events emitted by producers can be done\nautomatically by `GenStage`. Buffering the demand, instead, is a case that\nmust be explicitly considered by developers implementing producers.","ref":"GenStage.html#module-buffering"},{"type":"behaviour","title":"Buffering events - GenStage","doc":"Due to the concurrent nature of Elixir software, sometimes a producer\nmay dispatch events without consumers to send those events to. For example,\nimagine a `:consumer` B subscribes to `:producer` A. Next, the consumer B\nsends demand to A, which starts producing events to satisfy the demand.\nNow, if the consumer B crashes, the producer may attempt to dispatch the\nnow produced events but it no longer has a consumer to send those events to.\nIn such cases, the producer will automatically buffer the events until another\nconsumer subscribes. Note however, all of the events being consumed by\n`B` in its `c:handle_events/3` at the moment of the crash will be lost.\n\nThe buffer can also be used in cases where external sources only send\nevents in batches larger than asked for. For example, if you are\nreceiving events from an external source that only sends events\nin batches of 1000 and the internal demand is smaller than\nthat, the buffer allows you to always emit batches of 1000 events\neven when the consumer has asked for less. This tends to simplify\ncode in producers since they don't need to emit exactly as many events\nas the demand but can just emit events as they come.\n\nIn all of those cases when an event cannot be sent immediately by\na producer, the event will be automatically stored and sent the next\ntime consumers ask for events. The size of the buffer is configured\nvia the `:buffer_size` option returned by `c:init/1` and the default\nvalue is `10_000`. If the `:buffer_size` is exceeded, an error is logged.\nSee the documentation for `c:init/1` for more detailed information about\nthe `:buffer_size` option.","ref":"GenStage.html#module-buffering-events"},{"type":"behaviour","title":"Buffering demand - GenStage","doc":"In case consumers send demand and the producer is not yet ready to\nfill in the demand, producers must buffer the demand until data arrives.\n\nAs an example, let's implement a producer that broadcasts messages\nto consumers. For producers, we need to consider two scenarios:\n\n  1. what if events arrive and there are no consumers?\n  2. what if consumers send demand and there are not enough events?\n\nOne way to implement such a broadcaster is to simply rely on the internal\nbuffer available in `GenStage`, dispatching events as they arrive, as explained\nin the previous section:\n\n    defmodule Broadcaster do\n      use GenStage\n\n      @doc \"Starts the broadcaster.\"\n      def start_link() do\n        GenStage.start_link(__MODULE__, :ok, name: __MODULE__)\n      end\n\n      @doc \"Sends an event and returns only after the event is dispatched.\"\n      def sync_notify(event, timeout \\\\ 5000) do\n        GenStage.call(__MODULE__, {:notify, event}, timeout)\n      end\n\n      def init(:ok) do\n        {:producer, :ok, dispatcher: GenStage.BroadcastDispatcher}\n      end\n\n      def handle_call({:notify, event}, _from, state) do\n        {:reply, :ok, [event], state} # Dispatch immediately\n      end\n\n      def handle_demand(_demand, state) do\n        {:noreply, [], state} # We don't care about the demand\n      end\n    end\n\nBy always sending events as soon as they arrive, if there is any demand,\nwe will serve the existing demand, otherwise the event will be queued in\n`GenStage`'s internal buffer. In case events are being queued and not being\nconsumed, a log message will be emitted when we exceed the `:buffer_size`\nconfiguration. This behavior can be customized by implementing the optional\n`c:format_discarded/2` callback.\n\nWhile the implementation above is enough to solve the constraints above,\na more robust implementation would have tighter control over the events\nand demand by tracking this data locally, leaving the `GenStage` internal\nbuffer only for cases where consumers crash without consuming all data.\n\nTo handle such cases, we will use a two-element tuple as the broadcaster state\nwhere the first element is a queue and the second element is the pending\ndemand.  When events arrive and there are no consumers, we will store the\nevent in the queue alongside information about the process that broadcast\nthe event. When consumers send demand and there are not enough events, we will\nincrease the pending demand.  Once we have both data and demand, we\nacknowledge the process that has sent the event to the broadcaster and finally\nbroadcast the event downstream.\n\n    defmodule QueueBroadcaster do\n      use GenStage\n\n      @doc \"Starts the broadcaster.\"\n      def start_link() do\n        GenStage.start_link(__MODULE__, :ok, name: __MODULE__)\n      end\n\n      @doc \"Sends an event and returns only after the event is dispatched.\"\n      def sync_notify(event, timeout \\\\ 5000) do\n        GenStage.call(__MODULE__, {:notify, event}, timeout)\n      end","ref":"GenStage.html#module-buffering-demand"},{"type":"behaviour","title":"Callbacks - GenStage","doc":"def init(:ok) do\n        {:producer, {:queue.new, 0}, dispatcher: GenStage.BroadcastDispatcher}\n      end\n\n      def handle_call({:notify, event}, from, {queue, pending_demand}) do\n        queue = :queue.in({from, event}, queue)\n        dispatch_events(queue, pending_demand, [])\n      end\n\n      def handle_demand(incoming_demand, {queue, pending_demand}) do\n        dispatch_events(queue, incoming_demand + pending_demand, [])\n      end\n\n      defp dispatch_events(queue, 0, events) do\n        {:noreply, Enum.reverse(events), {queue, 0}}\n      end\n\n      defp dispatch_events(queue, demand, events) do\n        case :queue.out(queue) do\n          {{:value, {from, event}}, queue} ->\n            GenStage.reply(from, :ok)\n            dispatch_events(queue, demand - 1, [event | events])\n          {:empty, queue} ->\n            {:noreply, Enum.reverse(events), {queue, demand}}\n        end\n      end\n    end\n\nLet's also implement a consumer that automatically subscribes to the\nbroadcaster on `c:init/1`. The advantage of doing so on initialization\nis that, if the consumer crashes while it is supervised, the subscription\nis automatically re-established when the supervisor restarts it.\n\n    defmodule Printer do\n      use GenStage\n\n      @doc \"Starts the consumer.\"\n      def start_link() do\n        GenStage.start_link(__MODULE__, :ok)\n      end\n\n      def init(:ok) do\n        # Starts a permanent subscription to the broadcaster\n        # which will automatically start requesting items.\n        {:consumer, :ok, subscribe_to: [QueueBroadcaster]}\n      end\n\n      def handle_events(events, _from, state) do\n        for event <- events do\n          IO.inspect {self(), event}\n        end\n        {:noreply, [], state}\n      end\n    end\n\nWith the broadcaster in hand, now let's start the producer as well\nas multiple consumers:\n\n    # Start the producer\n    QueueBroadcaster.start_link()\n\n    # Start multiple consumers\n    Printer.start_link()\n    Printer.start_link()\n    Printer.start_link()\n    Printer.start_link()\n\nAt this point, all consumers must have sent their demand which we were not\nable to fulfill. Now by calling `QueueBroadcaster.sync_notify/1`, the event\nshall be broadcast to all consumers at once as we have buffered the demand\nin the producer:\n\n    QueueBroadcaster.sync_notify(:hello_world)\n\nIf we had called `QueueBroadcaster.sync_notify(:hello_world)` before any\nconsumer was available, the event would also have been buffered in our own\nqueue and served only when demand had been received.\n\nBy having control over the demand and queue, the broadcaster has\nfull control on how to behave when there are no consumers, when the\nqueue grows too large, and so forth.","ref":"GenStage.html#module-callbacks"},{"type":"behaviour","title":"Asynchronous work and handle_subscribe - GenStage","doc":"Both `:producer_consumer` and `:consumer` stages have been designed to do\ntheir work in the `c:handle_events/3` callback. This means that, after\n`c:handle_events/3` has been executed, both `:producer_consumer` and `:consumer`\nstages will immediately send demand upstream and ask for more items. It is\nassumed that events have been fully processed by `c:handle_events/3`.\n\nSuch default behaviour makes `:producer_consumer` and `:consumer` stages\nunfeasible for doing asynchronous work. However, given `GenStage` was designed\nto run with multiple consumers, it is not a problem to perform synchronous or\nblocking actions inside `handle_events/3` as you can then start multiple\nconsumers in order to max both CPU and IO usage as necessary.\n\nOn the other hand, if you must perform some work asynchronously,\n`GenStage` comes with an option that manually controls how demand\nis sent upstream, avoiding the default behaviour where demand is\nsent after `c:handle_events/3`. Such can be done by implementing\nthe `c:handle_subscribe/4` callback and returning `{:manual, state}`\ninstead of the default `{:automatic, state}`. Once the consumer mode\nis set to `:manual`, developers must use `GenStage.ask/3` to send\ndemand upstream when necessary.\n\nNote that `:max_demand` and `:min_demand` must be manually respected when\nasking for demand through `GenStage.ask/3`.\n\nFor example, the `ConsumerSupervisor` module processes events\nasynchronously by starting a process for each event and this is achieved by\nmanually sending demand to producers. `ConsumerSupervisor`\ncan be used to distribute work to a limited amount of\nprocesses, behaving similar to a pool where a new process is\nstarted for each event. See the `ConsumerSupervisor` docs for more\ninformation.\n\nSetting the demand to `:manual` in `c:handle_subscribe/4` is not\nonly useful for asynchronous work but also for setting up other\nmechanisms for back-pressure. As an example, let's implement a\nconsumer that is allowed to process a limited number of events\nper time interval. Those are often called rate limiters:\n\n    defmodule RateLimiter do\n      use GenStage\n\n      def init(_) do\n        # Our state will keep all producers and their pending demand\n        {:consumer, %{}}\n      end\n\n      def handle_subscribe(:producer, opts, from, producers) do\n        # We will only allow max_demand events every 5000 milliseconds\n        pending = opts[:max_demand] || 1000\n        interval = opts[:interval] || 5000\n\n        # Register the producer in the state\n        producers = Map.put(producers, from, {pending, interval})\n        # Ask for the pending events and schedule the next time around\n        producers = ask_and_schedule(producers, from)\n\n        # Returns manual as we want control over the demand\n        {:manual, producers}\n      end\n\n      def handle_cancel(_, from, producers) do\n        # Remove the producers from the map on unsubscribe\n        {:noreply, [], Map.delete(producers, from)}\n      end\n\n      def handle_events(events, from, producers) do\n        # Bump the amount of pending events for the given producer\n        producers = Map.update!(producers, from, fn {pending, interval} ->\n          {pending + length(events), interval}\n        end)\n\n        # Consume the events by printing them.\n        IO.inspect(events)\n\n        # A producer_consumer would return the processed events here.\n        {:noreply, [], producers}\n      end\n\n      def handle_info({:ask, from}, producers) do\n        # This callback is invoked by the Process.send_after/3 message below.\n        {:noreply, [], ask_and_schedule(producers, from)}\n      end\n\n      defp ask_and_schedule(producers, from) do\n        case producers do\n          %{^from => {pending, interval}} ->\n            # Ask for any pending events\n            GenStage.ask(from, pending)\n            # And let's check again after interval\n            Process.send_after(self(), {:ask, from}, interval)\n            # Finally, reset pending events to 0\n            Map.put(producers, from, {0, interval})\n          %{} ->\n            producers\n        end\n      end\n    end\n\nLet's subscribe the `RateLimiter` above to the\nproducer we have implemented at the beginning of the module\ndocumentation:\n\n    {:ok, a} = GenStage.start_link(A, 0)\n    {:ok, b} = GenStage.start_link(RateLimiter, :ok)\n\n    # Ask for 10 items every 2 seconds\n    GenStage.sync_subscribe(b, to: a, max_demand: 10, interval: 2000)\n\nAlthough the rate limiter above is a consumer, it could be made a\nproducer-consumer by changing `c:init/1` to return a `:producer_consumer`\nand then forwarding the events in `c:handle_events/3`.","ref":"GenStage.html#module-asynchronous-work-and-handle_subscribe"},{"type":"behaviour","title":"Callbacks - GenStage","doc":"`GenStage` is implemented on top of a `GenServer` with a few additions.\nBesides exposing all of the `GenServer` callbacks, it also provides\n`c:handle_demand/2` to be implemented by producers and `c:handle_events/3` to be\nimplemented by consumers, as shown above, as well as subscription-related\ncallbacks. Furthermore, all the callback responses have been modified to\npotentially emit events. See the callbacks documentation for more\ninformation.\n\nBy adding `use GenStage` to your module, Elixir will automatically\ndefine all callbacks for you except for the following ones:\n\n  * `c:init/1` - must be implemented to choose between `:producer`, `:consumer`, or `:producer_consumer` stages\n  * `c:handle_demand/2` - must be implemented by `:producer` stages\n  * `c:handle_events/3` - must be implemented by `:producer_consumer` and `:consumer` stages\n\n`use GenStage` also defines a `child_spec/1` function, allowing the\ndefined module to be put under a supervision tree in Elixir v1.5+.\nThe generated `child_spec/1` can be customized with the following options:\n\n  * `:id` - the child specification id, defaults to the current module\n  * `:start` - how to start the child process (defaults to calling `__MODULE__.start_link/1`)\n  * `:restart` - when the child should be restarted, defaults to `:permanent`\n  * `:shutdown` - how to shut down the child\n\nFor example:\n\n    use GenStage, restart: :transient, shutdown: 10_000\n\nSee the `Supervisor` docs for more information.\n\nAlthough this module exposes functions similar to the ones found in\nthe `GenServer` API, like `call/3` and `cast/2`, developers can also\nrely directly on GenServer functions such as `GenServer.multi_call/4`\nand `GenServer.abcast/3` if they wish to.","ref":"GenStage.html#module-callbacks-1"},{"type":"behaviour","title":"Name registration - GenStage","doc":"`GenStage` is bound to the same name registration rules as a `GenServer`.\nRead more about it in the `GenServer` docs.","ref":"GenStage.html#module-name-registration"},{"type":"behaviour","title":"Message protocol overview - GenStage","doc":"This section will describe the message protocol implemented\nby stages. By documenting these messages, we will allow\ndevelopers to provide their own stage implementations.","ref":"GenStage.html#module-message-protocol-overview"},{"type":"behaviour","title":"Back-pressure - GenStage","doc":"When data is sent between stages, it is done by a message\nprotocol that provides back-pressure. The first step is\nfor the consumer to subscribe to the producer. Each\nsubscription has a unique reference.\n\nOnce subscribed, the consumer may ask the producer for messages\nfor the given subscription. The consumer may demand more items\nwhenever it wants to. A consumer must never receive more data\nthan it has asked for from any given producer stage.\n\nA consumer may have multiple producers, where each demand is managed\nindividually (on a per-subscription basis). A producer may have multiple\nconsumers, where the demand and events are managed and delivered according to\na `GenStage.Dispatcher` implementation.","ref":"GenStage.html#module-back-pressure"},{"type":"behaviour","title":"Producer messages - GenStage","doc":"The producer is responsible for sending events to consumers\nbased on demand. These are the messages that consumers can\nsend to producers:\n\n  * `{:\"$gen_producer\", from :: {consumer_pid, subscription_tag}, {:subscribe, current, options}}` -\n    sent by the consumer to the producer to start a new subscription.\n\n    Before sending, the consumer MUST monitor the producer for clean-up\n    purposes in case of crashes. The `subscription_tag` is unique to\n    identify the subscription. It is typically the subscriber monitoring\n    reference although it may be any term.\n\n    Once sent, the consumer MAY immediately send demand to the producer.\n\n    The `current` field, when not `nil`, is a two-item tuple containing a\n    subscription that must be cancelled with the given reason before the\n    current one is accepted.\n\n    Once received, the producer MUST monitor the consumer. However, if\n    the subscription reference is known, it MUST send a `:cancel` message\n    to the consumer instead of monitoring and accepting the subscription.\n\n  * `{:\"$gen_producer\", from :: {consumer_pid, subscription_tag}, {:cancel, reason}}` -\n    sent by the consumer to cancel a given subscription.\n\n    Once received, the producer MUST send a `:cancel` reply to the\n    registered consumer (which may not necessarily be the one received\n    in the tuple above). Keep in mind, however, there is no guarantee\n    such messages can be delivered in case the producer crashes before.\n    If the pair is unknown, the producer MUST send an appropriate cancel\n    reply.\n\n  * `{:\"$gen_producer\", from :: {consumer_pid, subscription_tag}, {:ask, demand}}` -\n    sent by consumers to ask demand for a given subscription (identified\n    by `subscription_tag`).\n\n    Once received, the producer MUST send data up to the demand. If the\n    pair is unknown, the producer MUST send an appropriate cancel reply.","ref":"GenStage.html#module-producer-messages"},{"type":"behaviour","title":"Consumer messages - GenStage","doc":"The consumer is responsible for starting the subscription\nand sending demand to producers. These are the messages that\nproducers can send to consumers:\n\n  * `{:\"$gen_consumer\", from :: {producer_pid, subscription_tag}, {:cancel, reason}}` -\n    sent by producers to cancel a given subscription.\n\n    It is used as a confirmation for client cancellations OR\n    whenever the producer wants to cancel some upstream demand.\n\n  * `{:\"$gen_consumer\", from :: {producer_pid, subscription_tag}, events :: [event, ...]}` -\n    events sent by producers to consumers.\n\n    `subscription_tag` identifies the subscription. The third argument\n    is a non-empty list of events. If the subscription is unknown, the\n    events must be ignored and a cancel message must be sent to the producer.","ref":"GenStage.html#module-consumer-messages"},{"type":"function","title":"GenStage.ask/3","doc":"Asks the given demand to the producer.\n\n`producer_subscription` is the subscription this demand will be asked on; this\nterm could be for example stored in the stage when received in\n`c:handle_subscribe/4`.\n\nThe demand is a non-negative integer with the amount of events to\nask a producer for. If the demand is `0`, this function simply returns `:ok`\nwithout asking for data.\n\nThis function must only be used in the cases when a consumer\nsets a subscription to `:manual` mode in the `c:handle_subscribe/4`\ncallback.\n\nIt accepts the same options as `Process.send/3`, and returns the same value as\n`Process.send/3`.","ref":"GenStage.html#ask/3"},{"type":"function","title":"GenStage.async_info/2","doc":"Asynchronously queues an info message that is delivered after all\ncurrently buffered events.\n\nIf the stage is a consumer, it does not have buffered events, so the\nmessage is queued immediately.\n\nThis call returns `:ok` regardless if the info has been successfully\nqueued or not. It is typically called from the stage itself.","ref":"GenStage.html#async_info/2"},{"type":"function","title":"GenStage.async_resubscribe/4","doc":"Cancels `subscription_tag` with `reason` and resubscribe\nto the same stage with the given options.\n\nThis is useful in case you need to update the options in\nwhich you are currently subscribed to in a producer.\n\nThis function is async, which means it always returns\n`:ok` once the request is dispatched but without waiting\nfor its completion.","ref":"GenStage.html#async_resubscribe/4"},{"type":"function","title":"Options - GenStage.async_resubscribe/4","doc":"This function accepts the same options as `sync_subscribe/2`.","ref":"GenStage.html#async_resubscribe/4-options"},{"type":"function","title":"GenStage.async_subscribe/2","doc":"Asks the consumer to subscribe to the given producer asynchronously.\n\nThis function is async, which means it always returns\n`:ok` once the request is dispatched but without waiting\nfor its completion. This particular function is usually\ncalled from a stage's `c:init/1` callback.","ref":"GenStage.html#async_subscribe/2"},{"type":"function","title":"Options - GenStage.async_subscribe/2","doc":"This function accepts the same options as `sync_subscribe/2`.","ref":"GenStage.html#async_subscribe/2-options"},{"type":"function","title":"GenStage.call/3","doc":"Makes a synchronous call to the `stage` and waits for its reply.\n\nThe client sends the given `request` to the stage and waits until a reply\narrives or a timeout occurs. `c:handle_call/3` will be called on the stage\nto handle the request.\n\n`stage` can be any of the values described in the \"Name registration\"\nsection of the documentation for this module.","ref":"GenStage.html#call/3"},{"type":"function","title":"Timeouts - GenStage.call/3","doc":"`timeout` is an integer greater than zero which specifies how many\nmilliseconds to wait for a reply, or the atom `:infinity` to wait\nindefinitely. The default value is `5000`. If no reply is received within\nthe specified time, the function call fails and the caller exits. If the\ncaller catches the failure and continues running, and the stage is just late\nwith the reply, such reply may arrive at any time later into the caller's message\nqueue. The caller must in this case be prepared for this and discard any such\ngarbage messages that are two-element tuples with a reference as the first\nelement.","ref":"GenStage.html#call/3-timeouts"},{"type":"function","title":"GenStage.cancel/3","doc":"Cancels the given subscription on the producer.\n\nThe second argument is the cancellation reason. Once the\nproducer receives the request, a confirmation may be\nforwarded to the consumer (although there is no guarantee\nas the producer may crash for unrelated reasons before).\nThe consumer will react to the cancellation according to\nthe `:cancel` option given when subscribing. For example:\n\n    GenStage.cancel({pid, subscription}, :shutdown)\n\nwill cause the consumer to crash if the `:cancel` given\nwhen subscribing is `:permanent` (the default) but it\nwon't cause a crash in other modes. See the options in\n`sync_subscribe/3` for more information.\n\nThe `cancel` operation is an asynchronous request. The\nthird argument are same options as `Process.send/3`,\nallowing you to pass `:noconnect` or `:nosuspend` which\nis useful when working across nodes. This function returns\nthe same value as `Process.send/3`.","ref":"GenStage.html#cancel/3"},{"type":"function","title":"GenStage.cast/2","doc":"Sends an asynchronous request to the `stage`.\n\nThis function always returns `:ok` regardless of whether\nthe destination `stage` (or node) exists. Therefore it\nis unknown whether the destination stage successfully\nhandled the message.\n\n`c:handle_cast/2` will be called on the stage to handle\nthe request. In case the `stage` is on a node which is\nnot yet connected to the caller one, the call is going to\nblock until a connection happens.","ref":"GenStage.html#cast/2"},{"type":"callback","title":"GenStage.code_change/3","doc":"The same as `c:GenServer.code_change/3`.","ref":"GenStage.html#c:code_change/3"},{"type":"type","title":"GenStage.consumer_and_producer_consumer_option/0","doc":"Option values used by the `init*` common to `:consumer` and `:producer_consumer` types","ref":"GenStage.html#t:consumer_and_producer_consumer_option/0"},{"type":"type","title":"GenStage.consumer_option/0","doc":"Option values used by the `init*` functions when stage type is `:consumer`","ref":"GenStage.html#t:consumer_option/0"},{"type":"function","title":"GenStage.demand/1","doc":"Returns the demand mode for a producer.\n\nIt is either `:forward` or `:accumulate`. See `demand/2`.","ref":"GenStage.html#demand/1"},{"type":"function","title":"GenStage.demand/2","doc":"Sets the demand mode for a producer.\n\nWhen `:forward`, the demand is always forwarded to the `c:handle_demand/2`\ncallback. When `:accumulate`, both demand and events are accumulated until\nits mode is set to `:forward`. This is useful as a synchronization mechanism,\nwhere the demand is accumulated until all consumers are subscribed. Defaults\nto `:forward`.\n\nThis command is asynchronous.","ref":"GenStage.html#demand/2"},{"type":"function","title":"GenStage.estimate_buffered_count/2","doc":"Returns the estimated number of buffered items for a producer.","ref":"GenStage.html#estimate_buffered_count/2"},{"type":"callback","title":"GenStage.format_discarded/2","doc":"Invoked when items are discarded from the buffer.\n\nIt receives the number of excess (discarded) items from this invocation.\nThis callback returns a boolean that controls whether the default error log for discarded items is printed or not.\nReturn true to print the log, return false to skip the log.","ref":"GenStage.html#c:format_discarded/2"},{"type":"callback","title":"GenStage.format_status/2","doc":"The same as `c:GenServer.format_status/2`.","ref":"GenStage.html#c:format_status/2"},{"type":"type","title":"GenStage.from/0","doc":"The term that identifies a subscription associated with the corresponding producer/consumer.","ref":"GenStage.html#t:from/0"},{"type":"function","title":"GenStage.from_enumerable/2","doc":"Starts a producer stage from an enumerable (or stream).\n\nThis function will start a stage linked to the current process\nthat will take items from the enumerable when there is demand.\nSince streams are enumerables, we can also pass streams as\narguments (in fact, streams are the most common argument to\nthis function).\n\nThe enumerable is consumed in batches, retrieving `max_demand`\nitems the first time and then `max_demand - min_demand` the\nnext times. Therefore, for streams that cannot produce items\nthat fast, it is recommended to pass a lower `:max_demand`\nvalue as an option.\n\nIt is also expected the enumerable is able to produce the whole\nbatch on demand or terminate. If the enumerable is a blocking one,\nfor example, because it needs to wait for data from another source,\nit will block until the current batch is fully filled. GenStage and\nFlow were created exactly to address such issue. So if you have a\nblocking enumerable that you want to use in your Flow, then it must\nbe implemented with GenStage and integrated with `from_stages/2`.\n\nWhen the enumerable finishes or halts, the stage will exit with\n`:normal` reason. This means that, if a consumer subscribes to\nthe enumerable stage and the `:cancel` option is set to\n`:permanent`, which is the default, the consumer will also exit\nwith `:normal` reason. This behaviour can be changed by setting\nthe `:cancel` option to either `:transient` or `:temporary`\nat the moment of subscription as described in the `sync_subscribe/3`\ndocs.\n\nKeep in mind that streams that require the use of the process\ninbox to work won't behave as expected with this function since\nthe mailbox is controlled by the stage process itself. For example,\nyou must not pass the result `Task.async_stream/3` to this function.\nAs explained above, stateful or blocking enumerables are generally\ndiscouraged in `GenStage`, as `GenStage` was designed precisely to\nsupport exchange of data in such cases.","ref":"GenStage.html#from_enumerable/2"},{"type":"function","title":"Options - GenStage.from_enumerable/2","doc":"* `:link` - when false, does not link the stage to the current\n    process. Defaults to `true`.\n\n  * `:dispatcher` - the dispatcher responsible for handling demands.\n    Defaults to `GenStage.DemandDispatcher`. May be either an atom or\n    a tuple with the dispatcher and the dispatcher options.\n\n  * `:demand` - configures the demand to `:forward` or `:accumulate`\n    mode. See `c:init/1` and `demand/2` for more information.\n\n  * `:stacktrace` - the stacktrace of the function that started the\n    stream.\n\n  * `:on_cancel` - what happens when all consumers cancel. The default\n    is to keep the stream running. Set it to `:stop` to stop the producer.\n    To avoid race conditions, it is recommend to only set this option if\n    `:demand` is set to `:accumulate` and forwarded only after all consumers\n    subscribe\n\nAll other options that would be given for `start_link/3` are\nalso accepted.","ref":"GenStage.html#from_enumerable/2-options"},{"type":"callback","title":"GenStage.handle_call/3","doc":"Invoked to handle synchronous `call/3` messages.\n\n`call/3` will block until a reply is received (unless the call times out or\nnodes are disconnected).\n\n`request` is the request message sent by a `call/3`, `from` is a two-element tuple\ncontaining the caller's PID and a term that uniquely identifies the call, and\n`state` is the current state of the `GenStage`.\n\nReturning `{:reply, reply, [events], new_state}` sends the response `reply`\nto the caller after events are dispatched (or buffered) and continues the\nloop with new state `new_state`. In case you want to deliver the reply before\nprocessing events, use `reply/2` and return `{:noreply, [event],\nstate}`. Only `:producer` and `:producer_consumer` stages can return a\nnon-empty list of events.\n\nReturning `{:noreply, [event], new_state}` does not send a response to the\ncaller and processes the given events before continuing the loop with new\nstate `new_state`. The response must be sent with `reply/2`. Only `:producer` and\n`:producer_consumer` stages can return a non-empty list of events.\n\nHibernating is also supported as an atom to be returned from either\n`:reply` and `:noreply` tuples.\n\nReturning `{:stop, reason, reply, new_state}` stops the loop and `terminate/2`\nis called with reason `reason` and state `new_state`. Then the `reply` is sent\nas the response to the call and the process exits with reason `reason`.\n\nReturning `{:stop, reason, new_state}` is similar to\n`{:stop, reason, reply, new_state}` except that no reply is sent to the caller.\n\nIf this callback is not implemented, the default implementation by\n`use GenStage` will return `{:stop, {:bad_call, request}, state}`.","ref":"GenStage.html#c:handle_call/3"},{"type":"callback","title":"GenStage.handle_cancel/3","doc":"Invoked when a consumer is no longer subscribed to a producer.\n\nIt receives the cancellation reason, the `from` tuple representing the\ncancelled subscription and the state.  The `cancel_reason` will be a\n`{:cancel, _}` tuple if the reason for cancellation was a `GenStage.cancel/2`\ncall. Any other value means the cancellation reason was due to an EXIT.\n\nIf this callback is not implemented, the default implementation by\n`use GenStage` will return `{:noreply, [], state}`.\n\nReturn values are the same as `c:handle_cast/2`.","ref":"GenStage.html#c:handle_cancel/3"},{"type":"callback","title":"GenStage.handle_cast/2","doc":"Invoked to handle asynchronous `cast/2` messages.\n\n`request` is the request message sent by a `cast/2` and `state` is the current\nstate of the `GenStage`.\n\nReturning `{:noreply, [event], new_state}` dispatches the events and continues\nthe loop with new state `new_state`. Only `:producer` and `:producer_consumer`\nstages can return a non-empty list of events.\n\nReturning `{:noreply, [event], new_state, :hibernate}` is similar to\n`{:noreply, new_state}` except the process is hibernated before continuing the\nloop. See the return values for `c:GenServer.handle_call/3` for more information\non hibernation. Only `:producer` and `:producer_consumer` stages can return a\nnon-empty list of events.\n\nReturning `{:stop, reason, new_state}` stops the loop and `terminate/2` is\ncalled with the reason `reason` and state `new_state`. The process exits with\nreason `reason`.\n\nIf this callback is not implemented, the default implementation by\n`use GenStage` will return `{:stop, {:bad_cast, request}, state}`.","ref":"GenStage.html#c:handle_cast/2"},{"type":"callback","title":"GenStage.handle_demand/2","doc":"Invoked on `:producer` stages.\n\nThis callback is invoked on `:producer` stages with the demand from\nconsumers/dispatcher. The producer that implements this callback can use it to\ndispatch events.\n\nThe producer can do one of these:\n\n* Dispatch exactly as many events as `demand`.\n\n* Dispatch *more events* than `demand` - in this case, GenStage will\n  buffer the excess events. These events will be \"used\" by the\n  consumer/dispatcher next time demand is sent upstream. It's only\n  once the events in the buffer don't satisfy the demand anymore that\n  the `c:handle_demand/2` callback is invoked again. See the \"Buffering\"\n  section in the module documentation.\n\n* Dispatch less events than `demand` - in this case, the producer is\n  responsible for storing the demand (\"buffering demand\") and then emitting\n  events when they are available.\n\nSee the \"Demand\" section in the module documentation.\n\nThis callback must always be explicitly implemented by `:producer` stages.","ref":"GenStage.html#c:handle_demand/2"},{"type":"callback","title":"Examples - GenStage.handle_demand/2","doc":"","ref":"GenStage.html#c:handle_demand/2-examples"},{"type":"callback","title":"Demand with buffering - GenStage.handle_demand/2","doc":"In the following example, the producer emits enough events to at least\nsatisfy the demand, letting GenStage buffer any excess events:\n\n    @impl true\n    def handle_demand(demand, state) do\n      events = List.flatten(fetch_at_least_n_events(demand))\n      {:noreply, events, state}\n    end\n\n    defp fetch_at_least_n_events(demand) do\n      events = fetch_events()\n      demand_left = demand - length(events)\n\n      if demand_left > 0 do\n        [events | fetch_at_least_n_events(demand_left)]\n      else\n        events\n      end\n    end","ref":"GenStage.html#c:handle_demand/2-demand-with-buffering"},{"type":"callback","title":"Stopping when events are over - GenStage.handle_demand/2","doc":"In the next example, we implement the scenario where a producer can only\nproduce a limited number of events and should terminate gracefully when\nevents are not available anymore.\n\n    @impl true\n    def handle_demand(demand, state) do\n      case fetch_events() do\n        {:available, events} ->\n          {:noreply, events, state}\n\n        {:finished, last_events} ->\n          GenStage.async_info(self(), :events_finished)\n          {:noreply, last_events, state, :hibernate}\n      end\n    end\n\n    @impl true\n    def handle_info(:events_finished, state) do\n      {:stop, :normal, state}\n    end\n\nWe use the `async_info/2` function to send the producer itself a message\nthat gets delivered only after the `last_events` are dispatched. When handling\nsaid message, the producer stops with reason `:normal`, terminating gracefully.","ref":"GenStage.html#c:handle_demand/2-stopping-when-events-are-over"},{"type":"callback","title":"GenStage.handle_events/3","doc":"Invoked on `:producer_consumer` and `:consumer` stages to handle events.\n\nMust always be explicitly implemented by such types.\n\nReturn values are the same as `c:handle_cast/2`.","ref":"GenStage.html#c:handle_events/3"},{"type":"callback","title":"GenStage.handle_info/2","doc":"Invoked to handle all other messages.\n\n`message` is the message and `state` is the current state of the `GenStage`. When\na timeout occurs the message is `:timeout`.\n\nOnly `:producer` and `:producer_consumer` stages can return a non-empty list\nof events.\n\nIf this callback is not implemented, the default implementation by\n`use GenStage` will return `{:noreply, [], state}`.\n\nReturn values are the same as `c:handle_cast/2`.","ref":"GenStage.html#c:handle_info/2"},{"type":"callback","title":"GenStage.handle_subscribe/4","doc":"Invoked when a consumer subscribes to a producer.\n\nThis callback is invoked in both producers and consumers.\n`producer_or_consumer` will be `:producer` when this callback is\ninvoked on a consumer that subscribed to a producer, and `:consumer`\nif when this callback is invoked on producers a consumer subscribed to.\n\nFor consumers, successful subscriptions must return one of:\n\n  * `{:automatic, new_state}` - means the stage implementation will take care\n    of automatically sending demand to producers. This is the default.\n\n  * `{:manual, state}` - means that demand must be sent to producers\n    explicitly via `ask/3`. `:manual` subscriptions must be cancelled when\n    `c:handle_cancel/3` is called. `:manual` can be used when a special\n    behaviour is desired (for example, `ConsumerSupervisor` uses `:manual`\n    demand in its implementation).\n\nFor producers, successful subscriptions must always return\n`{:automatic, new_state}`. `:manual` mode is not supported.\n\nIf this callback is not implemented, the default implementation by\n`use GenStage` will return `{:automatic, state}`.","ref":"GenStage.html#c:handle_subscribe/4"},{"type":"callback","title":"Examples - GenStage.handle_subscribe/4","doc":"Let's see an example where we define this callback in a consumer that will use\n`:manual` mode. In this case, we'll store the subscription (`from`) in the\nstate in order to be able to use it later on when asking demand via `ask/3`.\n\n    def handle_subscribe(:producer, _options, from, state) do\n      new_state = %{state | subscription: from}\n      {:manual, new_state}\n    end","ref":"GenStage.html#c:handle_subscribe/4-examples"},{"type":"callback","title":"GenStage.init/1","doc":"Invoked when the server is started.\n\n`start_link/3` (or `start/3`) will block until this callback returns.\n`args` is the argument term (second argument) passed to `start_link/3`\n(or `start/3`).\n\nIn case of successful start, this callback must return a tuple\nwhere the first element is the stage type, which is one of:\n\n  * `:producer`\n  * `:consumer`\n  * `:producer_consumer` (if the stage is acting as both)\n\nFor example:\n\n    def init(args) do\n      {:producer, some_state}\n    end\n\nThe returned tuple may also contain 3 or 4 elements. The third\nelement may be the `:hibernate` atom or a set of options defined\nbelow.\n\nReturning `:ignore` will cause `start_link/3` to return `:ignore`\nand the process will exit normally without entering the loop or\ncalling `terminate/2`.\n\nReturning `{:stop, reason}` will cause `start_link/3` to return\n`{:error, reason}` and the process to exit with reason `reason`\nwithout entering the loop or calling `terminate/2`.","ref":"GenStage.html#c:init/1"},{"type":"callback","title":"Options - GenStage.init/1","doc":"This callback may return options. Some options are specific to\nthe chosen stage type while others are shared across all types.\n\n### `:producer` options\n\n  * `:demand` - when `:forward`, the demand is always forwarded to\n    the `c:handle_demand/2` callback. When `:accumulate`, demand is\n    accumulated until its mode is set to `:forward` via `demand/2`.\n    This is useful as a synchronization mechanism, where the demand\n    is accumulated until all consumers are subscribed. Defaults to\n    `:forward`.\n\n### `:producer` and `:producer_consumer` options\n\n  * `:buffer_size` - the size of the buffer to store events without\n    demand. Can be `:infinity` to signal no limit on the buffer size. Check\n    the \"Buffer events\" section of the module documentation. Defaults to\n    `10_000` for `:producer`, `:infinity` for `:producer_consumer`.\n\n  * `:buffer_keep` - returns whether the `:first` or `:last` entries\n    should be kept on the buffer in case the buffer size is exceeded.\n    Defaults to `:last`.\n\n  * `:dispatcher` - the dispatcher responsible for handling demands.\n    Defaults to `GenStage.DemandDispatcher`. May be either an atom\n    representing a dispatcher module or a two-element tuple with\n    the dispatcher module and the dispatcher options.\n\n### `:consumer` and `:producer_consumer` options\n\n  * `:subscribe_to` - a list of producers to subscribe to. Each element\n    represents either the producer module or a tuple with the producer module\n    and the subscription options (as defined in `sync_subscribe/2`).","ref":"GenStage.html#c:init/1-options"},{"type":"type","title":"GenStage.producer_and_producer_consumer_option/0","doc":"Option values used by the `init*` common to `:producer` and `:producer_consumer` types","ref":"GenStage.html#t:producer_and_producer_consumer_option/0"},{"type":"type","title":"GenStage.producer_consumer_option/0","doc":"Option values used by the `init*` functions when stage type is `:producer_consumer`","ref":"GenStage.html#t:producer_consumer_option/0"},{"type":"type","title":"GenStage.producer_only_option/0","doc":"Option values used by the `init*` specific to `:producer` type","ref":"GenStage.html#t:producer_only_option/0"},{"type":"type","title":"GenStage.producer_option/0","doc":"Option values used by the `init*` functions when stage type is `:producer`","ref":"GenStage.html#t:producer_option/0"},{"type":"function","title":"GenStage.reply/2","doc":"Replies to a client.\n\nThis function can be used to explicitly send a reply to a client that\ncalled `call/3` when the reply cannot be specified in the return value\nof `c:handle_call/3`.\n\n`client` must be the `from` argument (the second argument) accepted by\n`c:handle_call/3` callbacks. `reply` is an arbitrary term which will be given\nback to the client as the return value of the call.\n\nNote that `reply/2` can be called from any process, not just the `GenStage`\nthat originally received the call (as long as that `GenStage` communicated the\n`from` argument somehow).\n\nThis function always returns `:ok`.","ref":"GenStage.html#reply/2"},{"type":"function","title":"Examples - GenStage.reply/2","doc":"def handle_call(:reply_in_one_second, from, state) do\n      Process.send_after(self(), {:reply, from}, 1_000)\n      {:noreply, [], state}\n    end\n\n    def handle_info({:reply, from}, state) do\n      GenStage.reply(from, :one_second_has_passed)\n    end","ref":"GenStage.html#reply/2-examples"},{"type":"type","title":"GenStage.stage/0","doc":"The stage.","ref":"GenStage.html#t:stage/0"},{"type":"function","title":"GenStage.start/3","doc":"Starts a `GenStage` process without links (outside of a supervision tree).\n\nSee `start_link/3` for more information.","ref":"GenStage.html#start/3"},{"type":"function","title":"GenStage.start_link/3","doc":"Starts a `GenStage` process linked to the current process.\n\nThis is often used to start the `GenStage` as part of a supervision tree.\n\nOnce the server is started, the `init/1` function of the given `module` is\ncalled with `args` as its arguments to initialize the stage. To ensure a\nsynchronized start-up procedure, this function does not return until `init/1`\nhas returned.\n\nNote that a `GenStage` started with `start_link/3` is linked to the\nparent process and will exit in case of crashes from the parent. The `GenStage`\nwill also exit due to the `:normal` reason in case it is configured to trap\nexits in the `c:init/1` callback.","ref":"GenStage.html#start_link/3"},{"type":"function","title":"Options - GenStage.start_link/3","doc":"* `:name` - used for name registration as described in the \"Name\n    registration\" section of the module documentation\n\n  * `:debug` - if present, the corresponding function in the [`:sys`\n    module](https://erlang.org/doc/man/sys.html) is invoked\n\nThis function also accepts all the options accepted by\n`GenServer.start_link/3`.","ref":"GenStage.html#start_link/3-options"},{"type":"function","title":"Return values - GenStage.start_link/3","doc":"If the stage is successfully created and initialized, this function returns\n`{:ok, pid}`, where `pid` is the pid of the stage. If a process with the\nspecified name already exists, this function returns\n`{:error, {:already_started, pid}}` with the pid of that process.\n\nIf the `c:init/1` callback fails with `reason`, this function returns\n`{:error, reason}`. Otherwise, if `c:init/1` returns `{:stop, reason}`\nor `:ignore`, the process is terminated and this function returns\n`{:error, reason}` or `:ignore`, respectively.","ref":"GenStage.html#start_link/3-return-values"},{"type":"function","title":"GenStage.stop/3","doc":"Stops the stage with the given `reason`.\n\nThe `c:terminate/2` callback of the given `stage` will be invoked before\nexiting. This function returns `:ok` if the server terminates with the\ngiven reason; if it terminates with another reason, the call exits.\n\nThis function keeps OTP semantics regarding error reporting.\nIf the reason is any other than `:normal`, `:shutdown` or\n`{:shutdown, _}`, an error report is logged.","ref":"GenStage.html#stop/3"},{"type":"function","title":"GenStage.stream/2","doc":"Creates a stream that subscribes to the given producers\nand emits the appropriate messages.\n\nIt expects a list of producers to subscribe to. Each element\nrepresents the producer or a tuple with the producer and the\nsubscription options as defined in `sync_subscribe/2`:\n\n    GenStage.stream([{producer, max_demand: 100}])\n\nIf the producer process exits, the stream will exit with the same\nreason. If you want the stream to halt instead, set the cancel option\nto either `:transient` or `:temporary` as described in the\n`sync_subscribe/3` docs:\n\n    GenStage.stream([{producer, max_demand: 100, cancel: :transient}])\n\nOnce all producers are subscribed to, their demand is automatically\nset to `:forward` mode. See the `:demand` and `:producers`\noptions below for more information.\n\n`GenStage.stream/1` will \"hijack\" the inbox of the process\nenumerating the stream to subscribe and receive messages\nfrom producers. However it guarantees it won't remove or\nleave unwanted messages in the mailbox after enumeration\nunless one of the producers comes from a remote node.\nFor more information, read the \"Known limitations\" section\nbelow.","ref":"GenStage.html#stream/2"},{"type":"function","title":"Options - GenStage.stream/2","doc":"* `:demand` - sets the demand in producers to `:forward` or\n   `:accumulate` after subscription. Defaults to `:forward` so\n     the stream can receive items.\n\n  * `:producers` - the processes to set the demand to `:forward`\n    on initialization. It defaults to the processes being subscribed\n    to. Sometimes the stream is subscribing to a `:producer_consumer`\n    instead of a `:producer`, in such cases, you can set this option\n    to either an empty list or the list of actual producers so their\n    demand is properly set.","ref":"GenStage.html#stream/2-options"},{"type":"function","title":"Known limitations - GenStage.stream/2","doc":"### `from_enumerable/2`\n\nThis module also provides a function called `from_enumerable/2`\nwhich receives an enumerable (like a stream) and creates a stage\nthat emits data from the enumerable.\n\nGiven both `GenStage.from_enumerable/2` and `GenStage.stream/2`\nrequire the process inbox to send and receive messages, passing\nthe result of `from_enumerable/2` to this function will lead to\nunexpected behaviour, as `stream/2` will never receive the\nmessages it expects.","ref":"GenStage.html#stream/2-known-limitations"},{"type":"function","title":"Remote nodes - GenStage.stream/2","doc":"While it is possible to stream messages from remote nodes,\nsuch should be done with care. In particular, in case of\ndisconnections, there is a chance the producer will send\nmessages after the consumer receives its DOWN messages and\nthose will remain in the process inbox, violating the\ncommon scenario where `GenStage.stream/1` does not pollute\nthe caller inbox. In such cases, it is recommended to\nconsume such streams from a separate process which will be\ndiscarded after the stream is consumed.","ref":"GenStage.html#stream/2-remote-nodes"},{"type":"type","title":"GenStage.subscription_option/0","doc":"Option used by the `subscribe*` functions","ref":"GenStage.html#t:subscription_option/0"},{"type":"type","title":"GenStage.subscription_options/0","doc":"Options used by the `subscribe*` functions","ref":"GenStage.html#t:subscription_options/0"},{"type":"opaque","title":"GenStage.subscription_tag/0","doc":"The term that identifies a subscription.","ref":"GenStage.html#t:subscription_tag/0"},{"type":"function","title":"GenStage.sync_info/3","doc":"Queues an info message that is delivered after all currently buffered events.\n\nThis call is synchronous and will return after the stage has queued\nthe info message. The message will be eventually handled by the\n`c:handle_info/2` callback.\n\nIf the stage is a consumer, it does not have buffered events, so the\nmessaged is queued immediately.\n\nThis function will return `:ok` if the info message is successfully queued.","ref":"GenStage.html#sync_info/3"},{"type":"function","title":"GenStage.sync_resubscribe/5","doc":"Cancels `subscription_tag` with `reason` and resubscribe\nto the same stage with the given options.\n\nThis is useful in case you need to update the options in\nwhich you are currently subscribed to in a producer.\n\nThis function is sync, which means it will wait until the\nsubscription message is sent to the producer, although it\nwon't wait for the subscription confirmation.\n\nSee `sync_subscribe/2` for options and more information.","ref":"GenStage.html#sync_resubscribe/5"},{"type":"function","title":"GenStage.sync_subscribe/3","doc":"Asks the consumer to subscribe to the given producer synchronously.\n\nThis call is synchronous and will return after the called consumer\nsends the subscribe message to the producer. It does not, however,\nwait for the subscription confirmation. Therefore this function\nwill return before `c:handle_subscribe/4` is called in the consumer.\nIn other words, it guarantees the message was sent, but it does not\nguarantee a subscription has effectively been established.\n\nThis function will return `{:ok, subscription_tag}` as long as the\nsubscription message is sent. It will return `{:error, :not_a_consumer}`\nwhen the stage is not a consumer. `subscription_tag` is the second element\nof the two-element tuple that will be passed to `c:handle_subscribe/4`.","ref":"GenStage.html#sync_subscribe/3"},{"type":"function","title":"Options - GenStage.sync_subscribe/3","doc":"* `:cancel` - `:permanent` (default), `:transient` or `:temporary`.\n    When permanent, the consumer exits when the producer cancels or exits.\n    When transient, the consumer exits only if reason is not `:normal`,\n    `:shutdown`, or `{:shutdown, reason}`. When temporary, it never exits.\n    In case of exits, the same reason is used to exit the consumer.\n    In case of cancellations, the reason is wrapped in a `:cancel` tuple.\n\n  * `:min_demand` - the minimum demand for this subscription. See the module\n    documentation for more information.\n\n  * `:max_demand` - the maximum demand for this subscription. See the module\n    documentation for more information.\n\nAny other option is sent to the producer stage. This may be used by\ndispatchers for custom configuration. For example, if a producer uses\na `GenStage.BroadcastDispatcher`, an optional `:selector` function\nthat receives an event and returns a boolean limits this subscription to\nreceiving only those events where the selector function returns a truthy\nvalue:\n\n    GenStage.sync_subscribe(consumer,\n      to: producer,\n      selector: fn %{key: key} -> String.starts_with?(key, \"foo-\") end)","ref":"GenStage.html#sync_subscribe/3-options"},{"type":"callback","title":"GenStage.terminate/2","doc":"The same as `c:GenServer.terminate/2`.","ref":"GenStage.html#c:terminate/2"},{"type":"type","title":"GenStage.type/0","doc":"The supported stage types.","ref":"GenStage.html#t:type/0"},{"type":"module","title":"GenStage.BroadcastDispatcher","doc":"A dispatcher that accumulates demand from all consumers\nbefore broadcasting events to all of them.\n\nThis dispatcher guarantees that events are dispatched to all\nconsumers without exceeding the demand of any given consumer.","ref":"GenStage.BroadcastDispatcher.html"},{"type":"module","title":"The :selector option - GenStage.BroadcastDispatcher","doc":"If a producer uses `GenStage.BroadcastDispatcher`, its subscribers\ncan specify an optional `:selector` function that receives the event\nand returns a boolean in the subscription options.\n\nAssume `producer` and `consumer` are stages exchanging events of type\n`%{:key => String.t, any => any}`, then by calling\n\n    GenStage.sync_subscribe(consumer,\n      to: producer,\n      selector: fn %{key: key} -> String.starts_with?(key, \"foo-\") end)\n\n`consumer` will receive only the events broadcast from `producer`\nfor which the selector function returns a truthy value.\n\nThe `:selector` option can be specified in sync and async subscriptions,\nas well as in the `:subscribe_to` list in the return tuple of\n`c:GenStage.init/1`. For example:\n\n    def init(:ok) do\n      {:consumer, :ok, subscribe_to:\n        [{producer, selector: fn %{key: key} -> String.starts_with?(key, \"foo-\") end}]}\n    end","ref":"GenStage.BroadcastDispatcher.html#module-the-selector-option"},{"type":"module","title":"Demand while setting up - GenStage.BroadcastDispatcher","doc":"```\n              [Producer Consumer 1]\n             /                     \\\n[Producer] -                        - [Consumer]\n             \\                     /\n              [Producer Consumer 2]\n```\n\nWhen starting `Producer Consumer 1` before `Producer Consumer 2` (or even\nregular consumers), it is the first batch of events is only delivered to\n`Producer Consumer 1` since `Producer Consummer 2` is not registered yet.\n\nIt is therefore recommended to start the producer with\n`{:producer, state, demand: :accumulate}`, which pauses demand in the producers,\nand after all stages have been initialized, call `GenStage.demand/2` to resume\nthe producer.","ref":"GenStage.BroadcastDispatcher.html#module-demand-while-setting-up"},{"type":"module","title":"GenStage.DemandDispatcher","doc":"A dispatcher that sends batches to the highest demand.\n\nThis is the default dispatcher used by `GenStage`. In order\nto avoid greedy consumers, it is recommended that all consumers\nhave exactly the same maximum demand.","ref":"GenStage.DemandDispatcher.html"},{"type":"module","title":"Options - GenStage.DemandDispatcher","doc":"The demand dispatcher accepts the following options\non initialization:\n\n  * `:shuffle_demands_on_first_dispatch` - when `true`, shuffle the initial demands list\n    which is constructed on subscription before first dispatch. It prevents overloading\n    the first consumer on first dispatch. Defaults to `false`.\n\n  * `:max_demand` - the maximum demand expected on `GenStage.ask/3`.\n    Defaults to the first demand asked.","ref":"GenStage.DemandDispatcher.html#module-options"},{"type":"module","title":"Examples - GenStage.DemandDispatcher","doc":"To start a producer with demands shuffled on first dispatch:\n\n    {:producer, state, dispatcher: {GenStage.DemandDispatcher, shuffle_demands_on_first_dispatch: true}}","ref":"GenStage.DemandDispatcher.html#module-examples"},{"type":"behaviour","title":"GenStage.Dispatcher","doc":"This module defines the behaviour used by `:producer` and\n`:producer_consumer` to dispatch events.\n\nWhen using a `:producer` or `:producer_consumer`, the dispatcher\nmay be configured on init as follows:\n\n    {:producer, state, dispatcher: GenStage.BroadcastDispatcher}\n\nSome dispatchers may require options to be given on initialization,\nthose can be done with a tuple:\n\n    {:producer, state, dispatcher: {GenStage.PartitionDispatcher, partitions: 0..3}}\n\nElixir ships with the following dispatcher implementations:\n\n  * `GenStage.DemandDispatcher` - dispatches the given batch of\n    events to the consumer with the biggest demand in a FIFO\n    ordering. This is the default dispatcher.\n\n  * `GenStage.BroadcastDispatcher` - dispatches all events to all\n    consumers. The demand is only sent upstream once all consumers\n    ask for data.\n\n  * `GenStage.PartitionDispatcher` - dispatches all events to a\n    fixed amount of consumers that works as partitions according\n    to a hash function.\n\n>","ref":"GenStage.Dispatcher.html"},{"type":"behaviour","title":"Dispatcher State - GenStage.Dispatcher","doc":">\n>  Note that the Dispatcher state is stored separately from the state of the\n>  `GenStage` itself and neither side will have direct access to the state of\n>  the other.","ref":"GenStage.Dispatcher.html#module-dispatcher-state"},{"type":"callback","title":"GenStage.Dispatcher.ask/3","doc":"Called every time a consumer sends demand.\n\nThe demand will always be a positive integer (more than 0).\nThis callback must return the `actual_demand` as part of its\nreturn tuple. The returned demand is then sent to producers.\n\nIt is guaranteed the reference given in `from` points to a\nreference previously given in subscribe.","ref":"GenStage.Dispatcher.html#c:ask/3"},{"type":"callback","title":"GenStage.Dispatcher.cancel/2","doc":"Called every time a subscription is cancelled or the consumer goes down.\n\nIt is guaranteed the reference given in `from` points to a reference\npreviously given in subscribe.","ref":"GenStage.Dispatcher.html#c:cancel/2"},{"type":"callback","title":"GenStage.Dispatcher.dispatch/3","doc":"Called every time a producer wants to dispatch an event.\n\nThe events will always be a non empty list. This callback may\nreceive more events than previously asked and therefore must\nreturn events it cannot not effectively deliver as part of its\nreturn tuple. Any `leftover_events` will be stored by producers\nin their buffer.\n\nIt is important to emphasize that `leftover_events` can happen\nin any dispatcher implementation. After all, a consumer can\nsubscribe, ask for events and crash. Eventually the events\nthe consumer asked will be delivered while the consumer no longer\nexists, meaning they must be returned as left_over events until\nanother consumer subscribes.\n\nThis callback is responsible for sending events to consumer \nstages. In order to do so, you must store a `from` value from a \nprevious `ask/3` callback.  \n\nIt is recommended for these events to be sent with `Process.send/3` \nand the `[:noconnect]` option as the consumers are all monitored \nby the producer. For example:\n\n    Process.send(consumer, {:\"$gen_consumer\", {self(), consumer_ref}, events}, [:noconnect])","ref":"GenStage.Dispatcher.html#c:dispatch/3"},{"type":"callback","title":"GenStage.Dispatcher.info/2","doc":"Used to send an info message to the current process.\n\nIn case the dispatcher is doing buffering, the message must\nonly be sent after all currently buffered consumer messages are\ndelivered.","ref":"GenStage.Dispatcher.html#c:info/2"},{"type":"callback","title":"GenStage.Dispatcher.init/1","doc":"Called on initialization with the options given on `c:GenStage.init/1`.","ref":"GenStage.Dispatcher.html#c:init/1"},{"type":"type","title":"GenStage.Dispatcher.options/0","doc":"Options used by `init/1`","ref":"GenStage.Dispatcher.html#t:options/0"},{"type":"callback","title":"GenStage.Dispatcher.subscribe/3","doc":"Called every time the producer gets a new subscriber.","ref":"GenStage.Dispatcher.html#c:subscribe/3"},{"type":"module","title":"GenStage.PartitionDispatcher","doc":"A dispatcher that sends events according to partitions.\n\nThis dispatcher assumes that partitions are *evenly distributed*.\nSee the [\"Even distribution\"](#module-even-distribution) section for\nmore information.\n\nWhen multiple consumers subscribe to one partition, the producer\nbehaves like a `GenStage.DemandDispatcher` *within that partition*.","ref":"GenStage.PartitionDispatcher.html"},{"type":"module","title":"Options - GenStage.PartitionDispatcher","doc":"The partition dispatcher accepts the following options\non initialization:\n\n  * `:partitions` - the number of partitions to dispatch to. It may be\n    an integer with a total number of partitions, where each partition\n    is named from 0 up to `integer - 1`. For example, `partitions: 4`\n    will contain four partitions named `0`, `1`, `2` and `3`.\n\n    It may also be an *enumerable* that specifies the name of each partition.\n    For instance, `partitions: [:odd, :even]` will build two partitions,\n    named `:odd` and `:even`.\n\n  * `:hash` - the hashing algorithm. It's a function of type\n    `t:hash_function/0`, which receives the event and returns a tuple with two\n    elements: the event to be dispatched and the partition to dispatch it to.\n    The function can also return `:none`, in which case the event\n    is discarded. The partition must be one of the partitions specified in\n    `:partitions` above. The default uses:\n\n        fn event -> {event, :erlang.phash2(event, Enum.count(partitions))} end","ref":"GenStage.PartitionDispatcher.html#module-options"},{"type":"module","title":"Examples - GenStage.PartitionDispatcher","doc":"To start a producer with four partitions named `0`, `1`, `2`, and `3`:\n\n    {:producer, state, dispatcher: {GenStage.PartitionDispatcher, partitions: 0..3}}\n\nTo start a producer with two partitions named `:odd` and `:even`:\n\n    {:producer, state, dispatcher: {GenStage.PartitionDispatcher, partitions: [:odd, :even]}}","ref":"GenStage.PartitionDispatcher.html#module-examples"},{"type":"module","title":"Subscribe options - GenStage.PartitionDispatcher","doc":"When subscribing to a `GenStage` with a partition dispatcher the following\noption is required:\n\n  * `:partition` - the name of the partition. The partition must be one of\n    the partitions specified in `:partitions` above.","ref":"GenStage.PartitionDispatcher.html#module-subscribe-options"},{"type":"module","title":"Examples - GenStage.PartitionDispatcher","doc":"The partition function can be given either on `init`'s subscribe_to:\n\n    {:consumer, :ok, subscribe_to: [{producer, partition: 0}]}\n\nOr when calling `sync_subscribe`:\n\n    GenStage.sync_subscribe(consumer, to: producer, partition: 0)","ref":"GenStage.PartitionDispatcher.html#module-examples-1"},{"type":"module","title":"Even distribution - GenStage.PartitionDispatcher","doc":"This dispatcher assumes that partitions are *evenly distributed*.\nIf the data is uneven for long periods of time, then you may\nbuffer excessive data from busy partitions for long periods of\ntime. This happens because the producer is unable to distinguish\nfrom which particular consumer/partition demand arrives.\n\nLet's see an example. Imagine you have three consumers, each\nfor one partition: `A`, `B`, and `C`.\n\nLet's assume 60% of the data goes to `A`, 20% to `B`, and 20% to\n`C`. Let's also say that `max_demand` is `10` and `min_demand` is\n`5`. When the consumers initially request data (`10` events each),\nthe producer receives a total demand of `30`. A will receive `18` of\nthose (60%), while `B` and `C` receive `6` each (20%). After\nprocessing `5` events (the `min_demand`), each consumer requests\nadditional `5` events, for a total of `15` additional events. At\nthis point, that will be `9` additional elements for A, and 3\nadditional elements for B and C. At the end of these two rounds, we\nwill have:\n\n    A = 18 - 5 + 9 = 22 events\n    B = 6 - 5 + 3 = 4 events\n    C = 6 - 5 + 3 = 4 events\n\nFurthermore, as B and C request more items, A will only go further\nbehind. This behaviour is fine for spikes that should quickly\nresolve, but it can be problematic if the data is consistently uneven.","ref":"GenStage.PartitionDispatcher.html#module-even-distribution"},{"type":"type","title":"GenStage.PartitionDispatcher.hash_function/0","doc":"The type used for the function passed to the `:hash` option.","ref":"GenStage.PartitionDispatcher.html#t:hash_function/0"}],"proglang":"elixir","producer":{"name":"ex_doc","version":"0.38.2"},"content_type":"text/markdown"}